@@ -1,7 +1,8 @@
 VLSTAR <- function(y, exo = NULL, p = 1,
-                     m = 2, st = NULL, constant = TRUE, starting = NULL,
-                     n.combi = NULL, method = c('ML', 'NLS'), n.iter = 500,
-                      epsilon = 10^(-3)){
+                   m = 2, st = NULL, constant = TRUE, starting = NULL,
+                   n.combi = NULL, method = c('ML', 'NLS'), n.iter = 500,
+                   singlecgamma = FALSE,
+                   epsilon = 10^(-3)){
   y <- as.matrix(y)
   x <- exo
   method <- match.arg(method)
@@ -20,9 +21,9 @@ VLSTAR <- function(y, exo = NULL, p = 1,
     if(length(y[,1]) != length(as.matrix(x[,1])) | length(st) != length(as.matrix(x[,1])) | length(y[,1]) != length(st))
       stop('The length of the variables does not match!')
   }
-if(is.null(starting) & is.null(n.combi)){
-  stop('Starting values should be provided or estimated.')
-}
+  if(is.null(starting) & is.null(n.combi)){
+    stop('Starting values should be provided or estimated.')
+  }
 
   if(is.null(p) | p < 1){
     stop('Please, specify a valid lag order.')
@@ -40,8 +41,8 @@ if(is.null(starting) & is.null(n.combi)){
   ylag <- stats::lag(yt, -(1:p))
   ylag <- as.matrix(ylag)
   if(p>1){
-   lagg <- p-1
-  ylag <- ylag[-(1:lagg),]
+    lagg <- p-1
+    ylag <- ylag[-(1:lagg),]
   }
   y <- y[-c(1:p), ]
   ncoly <- ncol(y)
@@ -74,26 +75,31 @@ if(is.null(starting) & is.null(n.combi)){
   st <- st[(1+p):length(st)]
   ncolx <- ncol(x)
   param.init <- list()
-  param.init$gamma <- rep(1L, ncoly)
-  param.init$cg <- colMeans(y)
-  q <- ncol(x)-ncolylag
-  if (is.null(starting)){
+if(singlecgamma == TRUE){
+    param.init$gamma <- 1
+    param.init$c <- mean(y)
+    } else{
+      param.init$gamma <- rep(1L, ncoly)
+      param.init$c <- colMeans(y)
+      }
+ny <- ifelse(singlecgamma == TRUE, 1, ncoly)
+q <- ncol(x)-ncolylag
+if (is.null(starting)){
     COMBI <- list()
     GAMMA <- list()
     CJ <- list()
     for(t in 1:(m-1)){
       #Starting values for c and gamma
-      sumsq <- matrix(ncol = ncoly, nrow = (n.combi*n.combi))
-      gamma <- matrix(ncol = ncoly, nrow = n.combi)
-      cj <- matrix(ncol = ncoly, nrow = n.combi)
-      gamma[1, ] <- param.init$gamma
+      gamma <- matrix(ncol = ny, nrow = n.combi)
+      cj <- matrix(ncol = ny, nrow = n.combi)
+      gamma[1,] <- param.init$gamma
       cj[1,] <- param.init$c
       GAMMA[[t]] <- gamma*(1+rnorm(1, mean = 0, sd = 5))
       CJ[[t]] <- cj*(1+rnorm(1, mean = 0, sd = 5))
       #Grid for c and gamma
-      rangey <- matrix(nrow = ncoly, ncol = 2)
+      rangey <- matrix(nrow = ny, ncol = 2)
       combi <- list()
-      for(j in 1:ncoly){
+      for(j in 1:ny){
         rangey[j,] <- range(y[,j])
         CJ[[t]][2:n.combi,j] <- seq(from = rangey[j,1]*1.1, to = rangey[j,2], length.out = (n.combi-1))
         GAMMA[[t]][2:n.combi, j] <- seq(from = 0L, to = 100L, length.out = (n.combi-1))
@@ -109,7 +115,7 @@ if(is.null(starting) & is.null(n.combi)){
     for (l in 1:(n.combi*n.combi)){
       tryCatch({
         In <- diag(ncoly)
-        glog <- matrix(ncol=ncoly, nrow = nrowy)
+        glog <- matrix(ncol=ny, nrow = nrowy)
         GT <- list()
         Gtilde <- list()
         GG <- list()
@@ -121,10 +127,14 @@ if(is.null(starting) & is.null(n.combi)){
         ggxx <- matrix(ncol = (q+ncoly)*m*ncoly, nrow = (q+ncoly)*m*ncoly)
         for (i in 1:nrowx){
           for(t in 1:(m-1)){
-            for (j in 1 : ncoly){
+            for (j in 1 : ny){
               glog[i,j] <- (1+exp(-COMBI[[t]][[j]][l,2]*(st[i]-COMBI[[t]][[j]][l,1])))^(-1)
             }
-            Gt <- diag(glog[i,])
+            if(singlecgamma == TRUE){
+              Gt <- diag(rep(glog[i,1], ncoly))
+            }else{
+              Gt <- diag(glog[i,])
+            }
             GT[[t]] <- Gt
           }
           Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
@@ -154,11 +164,15 @@ if(is.null(starting) & is.null(n.combi)){
 
     #c and gamma minimazing the sum of squared residuals for each equation
 
-    cgamma <- matrix(nrow = ncoly, ncol = 2)
+    cgamma <- matrix(nrow = ny, ncol = 2)
     CGAMMA <- list()
     for (t in 1:(m-1)){
-      for(j in 1:ncoly){
-        cgamma[j,] <- as.matrix(COMBI[[t]][[j]][which.min(ssq[,j]),])
+      if(singlecgamma == T){
+        cgamma[j,] <-  as.matrix(COMBI[[t]][[j]][which.min(rowSums(ssq)),])
+      } else{
+        for(j in 1:ny){
+          cgamma[j,] <- as.matrix(COMBI[[t]][[j]][which.min(ssq[,j]),])
+      }
       }
       CGAMMA[[t]] <- cgamma
     }
@@ -177,258 +191,274 @@ if(is.null(starting) & is.null(n.combi)){
   }
 
   if(method == 'ML'){
-  #NLS Estimation of Bhat and Omegahat to be used in the first iteration of maximum likelihood
-  In <- diag(ncoly)
-  glog <- matrix(ncol=ncoly, nrow = nrowy)
-  GT <- list()
-  Gtilde <- list()
-  GG <- list()
-  XX <- list()
-  GGXX <- list()
-  XY <- list()
-  XYG <- list()
-  kro <- list()
-  ggxx <- matrix(ncol = (q+ncoly)*m*ncoly, nrow = (q+ncoly)*m*ncoly)
-  for (i in 1:nrowx){
-    for(t in 1:(m-1)){
-      for (j in 1 : ncoly){
-        glog[i,j] <- (1L+exp(-PARAM[[t]][j,1]*(st[i]-PARAM[[t]][j,2])))^(-1)
-      }
-      Gt <- diag(glog[i,])
-      GT[[t]] <- Gt
-    }
-    Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
-    GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
-    XX[[i]] <- x[i,] %*%t(x[i,])
-    GGXX[[i]] <- kronecker(GG[[i]], XX[[i]])
-    XY[[i]] <- x[i, ]%*%t(y[i,])
-    XYG[[i]] <- vec(XY[[i]]%*%t(Gtilde[[i]]))
-    kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
-  }
-  ggxx <- Reduce(`+`, GGXX)/nrowx
-  M <- t(do.call("cbind", kro))
-  Y <- vec(t(y))
-  #Estimated coefficients
-  Bhat <- ginv(t(M)%*%M)%*%t(M)%*%Y
-  BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
-  resi <- list()
-  Ehat1 <- matrix(NA, ncol = ncoly, nrow = nrowy)
-  for (i in 1:nrowx){
-    resi[[i]] <- y[i, ] - t(Gtilde[[i]])%*%t(BB)%*%x[i,]
-  }
-  Ehat1 <- t(do.call("cbind", resi))
-  #Estimated covariance matrix
-  Omegahat <- (t(Ehat1)%*%Ehat1)/(nrowy)
-
-  #BB0 <- BB
-  #Omega0 <- Omegahat
-
-  #Log-likelihood to be optimized
-  loglike <- function(param){
-    gamma <- param[1:((m-1)*ncoly)]
-    c <- param[(ncoly*(m-1)+1):length(param)]
-    gamma1 <- matrix(gamma, ncol = (m-1))
-    c1 <- matrix(c, ncol = (m-1))
-    glog <- matrix(ncol=ncoly, nrow = nrowy)
-    GT <- list()
-    Gtilde <- list()
-    dify <- matrix(ncol = 1, nrow = nrow(y))
-    ncoly <- dim(y)[2]
+    #NLS Estimation of Bhat and Omegahat to be used in the first iteration of maximum likelihood
     In <- diag(ncoly)
-    for (z in 1:nrow(y)){
-      for(t in 1:(m-1)){
-        for (o in 1:ncoly){
-          gammao <- gamma[o]
-          co <- c[o]
-          glog[z,o] <- (1L+exp(-gammao*(st[z]-co)))^(-1)}
-        Gt <- diag(glog[z,])
-        GT[[t]] <- Gt
-      }
-      Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
-      dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
-    }
-    sumdif <- sum(dify)
-    logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
-    return(-logll)
-  }
-
-  #Maximum likelihood calculation
-  loglike1 <- function(BB, param, Omega){
-    gamma <- param[1:((m-1)*ncoly)]
-    c <- param[(ncoly*(m-1)+1):length(param)]
-    gamma1 <- matrix(gamma, ncol = (m-1))
-    c1 <- matrix(c, ncol = (m-1))
-    glog <- matrix(ncol=ncoly, nrow = nrowy)
-    GT <- list()
-    Gtilde <- list()
-    dify <- matrix(ncol = 1, nrow = nrow(y))
-    ncoly <- dim(y)[2]
-    In <- diag(ncoly)
-    for (z in 1:nrow(y)){
-      for(t in 1:(m-1)){
-        for (o in 1:ncoly){
-          gammao <- gamma[o]
-          co <- c[o]
-          glog[z,o] <- (1L+exp(-gammao*(st[z]-co)))^(-1)}
-        Gt <- diag(glog[z,])
-        GT[[t]] <- Gt
-      }
-      Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
-      dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
-    }
-    sumdif <- sum(dify)
-    logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
-    return(-logll)
-  }
-
-  #Inizialization of iter
-  iter <- 0
-  ll0 <- 10^(4)
-  epsi <- epsilon
-  err <- list()
-  loglik1 <- NULL
-  bbhat <- list()
-  omega <- list()
-  errdif <- 10^5
-  PARAM1 <- list()
-  for(t in 1:(m-1)){
-    PARAM1[[t]] <- as.data.frame(PARAM[[t]])
-  }
-  param <- rbindlist(PARAM1)
-  param <- vec(as.matrix(param))
-
-  message('Maximum likelihood estimation\n')
-
-  #Convergence algorithm
-  #1. Maximum likelihood estimation of gamma and c with NLS estimates of Bhat and Omegahat
-  #2. Maximum likelihood estimation of Bhat with new values of gamma and c
-  #3. Convergence check
-  #4. 1-2-3 until convergence
-  while (iter < n.iter & errdif > epsi){
-    Sys.sleep(0)
-    iter <- iter+1
-    #Parameters
-    low1 <- replicate(ncoly, 0)
-    #1.Maximum likelihood estimation of gamma and c
-    param1 <- optim(par = param, fn = loglike, lower = c(low1, apply(y, 2, min)),
-                           method="L-BFGS-B")
-
-    cgam1 <- matrix(param1$par, ncol = 2, nrow = ncoly)
-
-    #2.Maximum likelihood estimation of Bhat with new values of gamma and c
-    glog <- matrix(ncol=ncoly, nrow = nrowy)
+    glog <- matrix(ncol=ny, nrow = nrowy)
     GT <- list()
     Gtilde <- list()
     GG <- list()
     XX <- list()
+    GGXX <- list()
     XY <- list()
-    XYOG <- list()
+    XYG <- list()
     kro <- list()
-    PsiOmegaPsi <- list()
     ggxx <- matrix(ncol = (q+ncoly)*m*ncoly, nrow = (q+ncoly)*m*ncoly)
     for (i in 1:nrowx){
       for(t in 1:(m-1)){
-        for (j in 1 : ncoly){
-          glog[i,j] <- (1+exp(-cgam1[j,1]*(st[i]-cgam1[j,2])))^(-1)
+        for (j in 1 : ny){
+          glog[i,j] <- (1L+exp(-PARAM[[t]][j,1]*(st[i]-PARAM[[t]][j,2])))^(-1)
+        }
+        if(singlecgamma == TRUE){
+          Gt <- diag(rep(glog[i,1], ncoly))
+        }else{
+          Gt <- diag(glog[i,])
         }
-        Gt <- diag(glog[i,])
         GT[[t]] <- Gt
       }
       Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
       GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
       XX[[i]] <- x[i,] %*%t(x[i,])
+      GGXX[[i]] <- kronecker(GG[[i]], XX[[i]])
       XY[[i]] <- x[i, ]%*%t(y[i,])
-      XYOG[[i]] <- vec(XY[[i]]%*%ginv(Omegahat)%*%t(Gtilde[[i]]))
-      PsiOmegaPsi[[i]] <- Gtilde[[i]]%*%ginv(Omegahat)%*%t(Gtilde[[i]])
-      kro[[i]] <- kronecker(PsiOmegaPsi[[i]], XX[[i]])
+      XYG[[i]] <- vec(XY[[i]]%*%t(Gtilde[[i]]))
+      kro[[i]] <- kronecker(Gtilde[[i]], x[i,])
     }
-    xyog <- Reduce(`+`, XYOG)/nrowy
-    kroxx <- Reduce(`+`, kro)/nrowy
-    Bhat <- t(t(xyog)%*%kroxx)
+    ggxx <- Reduce(`+`, GGXX)/nrowx
+    M <- t(do.call("cbind", kro))
+    Y <- vec(t(y))
+    #Estimated coefficients
+    Bhat <- ginv(t(M)%*%M)%*%t(M)%*%Y
     BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
     resi <- list()
-    fitte <- matrix(nrow = nrowy, ncol = ncoly)
     Ehat1 <- matrix(NA, ncol = ncoly, nrow = nrowy)
     for (i in 1:nrowx){
       resi[[i]] <- y[i, ] - t(Gtilde[[i]])%*%t(BB)%*%x[i,]
-      fitte[i,] <- t(t(Gtilde[[i]])%*%t(BB)%*%x[i,])
     }
     Ehat1 <- t(do.call("cbind", resi))
+    #Estimated covariance matrix
     Omegahat <- (t(Ehat1)%*%Ehat1)/(nrowy)
 
-    #3. Convergence check
-    ll1 <- loglike1(BB, param1$par, Omega = Omegahat)
-    if((ll1-ll0)>0){
-      param <- as.matrix(vec(cgam1))
+    #BB0 <- BB
+    #Omega0 <- Omegahat
+
+    #Log-likelihood to be optimized
+    loglike <- function(param){
+      gamma <- param[1:((m-1)*ny)]
+      c <- param[(ny*(m-1)+1):length(param)]
+      gamma1 <- matrix(gamma, ncol = (m-1))
+      c1 <- matrix(c, ncol = (m-1))
+      glog <- matrix(ncol=ny, nrow = nrowy)
+      GT <- list()
+      Gtilde <- list()
+      dify <- matrix(ncol = 1, nrow = nrow(y))
+      ncoly <- dim(y)[2]
+      In <- diag(ncoly)
+      for (z in 1:nrow(y)){
+        for(t in 1:(m-1)){
+          for (o in 1:ny){
+            gammao <- gamma[o]
+            co <- c[o]
+            glog[z,o] <- (1L+exp(-gammao*(st[z]-co)))^(-1)}
+          if(singlecgamma == TRUE){
+            Gt <- diag(rep(glog[z,1], ncoly))
+          }else{
+            Gt <- diag(glog[z,])
+          }
+          GT[[t]] <- Gt
+        }
+        Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
+        dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
+      }
+      sumdif <- sum(dify)
+      logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
+      return(-logll)
     }
-    err <- abs(ll1 - ll0)
-    ll0 <- ll1
-    loglik1[iter] <- ll1
-    bbhat[[iter]] <- BB
-    omega[[iter]] <- Omegahat
 
-    message(paste("iteration", iter, "complete\n"))
+    #Maximum likelihood calculation
+    loglike1 <- function(BB, param, Omega){
+      gamma <- param[1:((m-1)*ny)]
+      c <- param[(ny*(m-1)+1):length(param)]
+      gamma1 <- matrix(gamma, ncol = (m-1))
+      c1 <- matrix(c, ncol = (m-1))
+      glog <- matrix(ncol=ny, nrow = nrowy)
+      GT <- list()
+      Gtilde <- list()
+      dify <- matrix(ncol = 1, nrow = nrow(y))
+      ncoly <- dim(y)[2]
+      In <- diag(ncoly)
+      for (z in 1:nrow(y)){
+        for(t in 1:(m-1)){
+          for (o in 1:ny){
+            gammao <- gamma[o]
+            co <- c[o]
+            glog[z,o] <- (1L+exp(-gammao*(st[z]-co)))^(-1)}
+          if(singlecgamma == TRUE){
+            Gt <- diag(rep(glog[z,1], ncoly))
+          }else{
+            Gt <- diag(glog[z,])
+          }
+          GT[[t]] <- Gt
+        }
+        Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
+        dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omega)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
+      }
+      sumdif <- sum(dify)
+      logll <- -(nrowy*log(det(Omega))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
+      return(-logll)
+    }
 
-    print(ll0)
+    #Inizialization of iter
+    iter <- 0
+    ll0 <- 10^(4)
+    epsi <- epsilon
+    err <- list()
+    loglik1 <- NULL
+    bbhat <- list()
+    omega <- list()
+    errdif <- 10^5
+    PARAM1 <- list()
+    for(t in 1:(m-1)){
+      PARAM1[[t]] <- as.data.frame(PARAM[[t]])
+    }
+    param <- rbindlist(PARAM1)
+    param <- vec(as.matrix(param))
 
-    if (err<epsi | iter == n.iter) message('Converged\n')}
+    message('Maximum likelihood estimation\n')
 
-  #Bhat1 <- bbhat[[which.min(loglik1)]]
-  residuals1 <- t(do.call("cbind", resi))
-  varhat <- diag(omega[[iter]])
-  bb1 <- bbhat[[iter]][,1:ncoly]
-  bb2 <- list()
-  for(t in 1:(m-1)){
-    bb2[[t]] <- as.data.frame(bbhat[[iter]][,(ncoly*(t-1)+1):(ncoly*t)] +
-                                bbhat[[iter]][,(ncoly*(t)+1):(ncoly*(t+1))])
-  }
-  bb4 <- as.matrix(rbindlist(bb2))
-  BBhat <- rbind(bb1, bb4)
-  colnames(cgam1) <- c('gamma', 'c')
-  covbb <- matrix(nrow = m*ncolx, ncol = ncoly)
-  ttest <- matrix(nrow = m*ncolx, ncol = ncoly)
-  pval <- matrix(nrow = m*ncolx, ncol = ncoly)
-  ee <- matrix(nrow = m*ncolx, ncol = ncoly)
-  for (j in 1 : ncoly){
-    #covbb[,j] <- diag(ginv(t(x[[j]])%*%XX[[j]])*sqrt(varhat[j]))
-    covbb[,j] <- sqrt(diag(ginv(t(x) %*%x))*varhat[j])
-    ttest[,j] <- BBhat[,j]/covbb[,j]
-    pval[,j] <- 2*pt(abs(ttest[,j]),df=(nrowy*m-m*(ncolx)), lower.tail = FALSE)
-  }
+    #Convergence algorithm
+    #1. Maximum likelihood estimation of gamma and c with NLS estimates of Bhat and Omegahat
+    #2. Maximum likelihood estimation of Bhat with new values of gamma and c
+    #3. Convergence check
+    #4. 1-2-3 until convergence
+    while (iter < n.iter & errdif > epsi){
+      Sys.sleep(0)
+      iter <- iter+1
+      #Parameters
+      low1 <- replicate(ncoly, 0)
+      #1.Maximum likelihood estimation of gamma and c
+      param1 <- optim(par = param, fn = loglike, lower = c(low1, apply(y, 2, min)),
+                      method="L-BFGS-B")
 
-  loglike2 <- function(y, resid1, omega){
-    nrowy <- nrow(as.matrix(y))
-    logll <- -(nrowy/2)*log(2*pi) -(nrowy/2)*log(omega) - (t(resid1)%*%resid1)/(2*omega)
-    return(logll)
-  }
-  residui <- residuals1
-  omega1 <- diag(omega[[iter]])
-  k <- nrow(BBhat)
-  ll2 <- NULL
-  AIC1 <- NULL
-  BIC1 <- NULL
-  for (l in 1:ncoly){
-    ll2[l] <- loglike2(y[,l], residui[,l], omega1[l])
-    AIC1[l] <- 2*k - 2*ll2[l]
-    BIC1[l] <- -2*ll2[l] + k*log(nrowy)
-  }
-  names1 <- list()
-  for(j in 1:m){
-    names1[[j]] <- as.data.frame(paste(colnames(x), 'm_', j))
-  }
-  names1 <- as.matrix(rbindlist(names1))
-  rownames(BBhat) <- names1
-  colnames(BBhat) <- colnames(y)
-  modeldata <- list(y, x)
-  fitte <- fitte[!is.na(fitte[,1]),]
-  results <- list(BBhat, covbb, ttest, pval, cgam1, omega[[iter]], fitte, residuals1, ll1, ll2, AIC1, BIC1, Gt, modeldata, BB, m, p,
-                  st, y, exo, constant)
-  names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Gammac', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
-                      'BIC', 'Gtilde', 'Data', 'B', 'm', 'p', 'st', 'yoriginal', 'exo', 'constant')
-  }else{
+      cgam1 <- matrix(param1$par, ncol = 2, nrow = (ny*(m-1)))
+
+      #2.Maximum likelihood estimation of Bhat with new values of gamma and c
+      glog <- matrix(ncol=ny, nrow = nrowy)
+      GT <- list()
+      Gtilde <- list()
+      GG <- list()
+      XX <- list()
+      XY <- list()
+      XYOG <- list()
+      kro <- list()
+      PsiOmegaPsi <- list()
+      ggxx <- matrix(ncol = (q+ncoly)*m*ncoly, nrow = (q+ncoly)*m*ncoly)
+      for (i in 1:nrowx){
+        for(t in 1:(m-1)){
+          for (j in 1 : ny){
+            glog[i,j] <- (1+exp(-cgam1[j,1]*(st[i]-cgam1[j,2])))^(-1)
+          }
+          if(singlecgamma == TRUE){
+            Gt <- diag(rep(glog[i,1], ncoly))
+          }else{
+            Gt <- diag(glog[i,])
+          }
+          GT[[t]] <- Gt
+        }
+        Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
+        GG[[i]] <- Gtilde[[i]]%*%t(Gtilde[[i]])
+        XX[[i]] <- x[i,] %*%t(x[i,])
+        XY[[i]] <- x[i, ]%*%t(y[i,])
+        XYOG[[i]] <- vec(XY[[i]]%*%ginv(Omegahat)%*%t(Gtilde[[i]]))
+        PsiOmegaPsi[[i]] <- Gtilde[[i]]%*%ginv(Omegahat)%*%t(Gtilde[[i]])
+        kro[[i]] <- kronecker(PsiOmegaPsi[[i]], XX[[i]])
+      }
+      xyog <- Reduce(`+`, XYOG)/nrowy
+      kroxx <- Reduce(`+`, kro)/nrowy
+      Bhat <- t(t(xyog)%*%kroxx)
+      BB <- invvec(Bhat, ncol = (m*ncoly), nrow = (ncolylag + q))
+      resi <- list()
+      fitte <- matrix(nrow = nrowy, ncol = ncoly)
+      Ehat1 <- matrix(NA, ncol = ncoly, nrow = nrowy)
+      for (i in 1:nrowx){
+        resi[[i]] <- y[i, ] - t(Gtilde[[i]])%*%t(BB)%*%x[i,]
+        fitte[i,] <- t(t(Gtilde[[i]])%*%t(BB)%*%x[i,])
+      }
+      Ehat1 <- t(do.call("cbind", resi))
+      Omegahat <- (t(Ehat1)%*%Ehat1)/(nrowy)
+
+      #3. Convergence check
+      ll1 <- loglike1(BB, param1$par, Omega = Omegahat)
+      if((ll1-ll0)>0){
+        param <- as.matrix(vec(cgam1))
+      }
+      err <- abs(ll1 - ll0)
+      ll0 <- ll1
+      loglik1[iter] <- ll1
+      bbhat[[iter]] <- BB
+      omega[[iter]] <- Omegahat
+
+      message(paste("iteration", iter, "complete\n"))
+
+      print(ll0)
+
+      if (err<epsi | iter == n.iter) message('Converged\n')}
+
+    #Bhat1 <- bbhat[[which.min(loglik1)]]
+    residuals1 <- t(do.call("cbind", resi))
+    varhat <- diag(omega[[iter]])
+    bb1 <- bbhat[[iter]][,1:ncoly]
+    bb2 <- list()
+    for(t in 1:(m-1)){
+      bb2[[t]] <- as.data.frame(bbhat[[iter]][,(ncoly*(t-1)+1):(ncoly*t)] +
+                                  bbhat[[iter]][,(ncoly*(t)+1):(ncoly*(t+1))])
+    }
+    bb4 <- as.matrix(rbindlist(bb2))
+    BBhat <- rbind(bb1, bb4)
+    colnames(cgam1) <- c('gamma', 'c')
+    covbb <- matrix(nrow = m*ncolx, ncol = ncoly)
+    ttest <- matrix(nrow = m*ncolx, ncol = ncoly)
+    pval <- matrix(nrow = m*ncolx, ncol = ncoly)
+    ee <- matrix(nrow = m*ncolx, ncol = ncoly)
+    for (j in 1 : ncoly){
+      #covbb[,j] <- diag(ginv(t(x[[j]])%*%XX[[j]])*sqrt(varhat[j]))
+      covbb[,j] <- sqrt(diag(ginv(t(x) %*%x))*varhat[j])
+      ttest[,j] <- BBhat[,j]/covbb[,j]
+      pval[,j] <- 2*pt(abs(ttest[,j]),df=(nrowy*m-m*(ncolx)), lower.tail = FALSE)
+    }
+
+    loglike2 <- function(y, resid1, omega){
+      nrowy <- nrow(as.matrix(y))
+      logll <- -(nrowy/2)*log(2*pi) -(nrowy/2)*log(omega) - (t(resid1)%*%resid1)/(2*omega)
+      return(logll)
+    }
+    residui <- residuals1
+    omega1 <- diag(omega[[iter]])
+    k <- nrow(BBhat)
+    ll2 <- NULL
+    AIC1 <- NULL
+    BIC1 <- NULL
+    for (l in 1:ncoly){
+      ll2[l] <- loglike2(y[,l], residui[,l], omega1[l])
+      AIC1[l] <- 2*k - 2*ll2[l]
+      BIC1[l] <- -2*ll2[l] + k*log(nrowy)
+    }
+    names1 <- list()
+    for(j in 1:m){
+      names1[[j]] <- as.data.frame(paste(colnames(x), 'm_', j))
+    }
+    names1 <- as.matrix(rbindlist(names1))
+    rownames(BBhat) <- names1
+    colnames(BBhat) <- colnames(y)
+    modeldata <- list(y, x)
+    fitte <- fitte[!is.na(fitte[,1]),]
+    results <- list(BBhat, covbb, ttest, pval, cgam1, omega[[iter]], fitte, residuals1, ll1, ll2, AIC1, BIC1, Gtilde, modeldata, BB, m, p,
+                    st, y, exo, constant, method, singlecgamma)
+    names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Gammac', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
+                        'BIC', 'Gtilde', 'Data', 'B', 'm', 'p', 'st', 'yoriginal', 'exo', 'constant', 'method', 'singlecgamma')
+  } else{
     #NLS Estimation of Bhat and Omegahat to be used in the first iteration of minimizing Qt
     In <- diag(ncoly)
-    glog <- matrix(ncol=ncoly, nrow = nrowy)
+    glog <- matrix(ncol=ny, nrow = nrowy)
     GT <- list()
     Gtilde <- list()
     GG <- list()
@@ -440,10 +470,14 @@ if(is.null(starting) & is.null(n.combi)){
     ggxx <- matrix(ncol = (q+ncoly)*m*ncoly, nrow = (q+ncoly)*m*ncoly)
     for (i in 1:nrowx){
       for (t in 1:(m-1)){
-        for (j in 1 : ncoly){
+        for (j in 1 : ny){
           glog[i,j] <- (1L+exp(-PARAM[[t]][j,1]*(st[i]-PARAM[[t]][j,2])))^(-1)
         }
-        Gt <- diag(glog[i,])
+        if(singlecgamma == TRUE){
+          Gt <- diag(rep(glog[i,1], ncoly))
+        }else{
+          Gt <- diag(glog[i,])
+        }
         GT[[t]] <- Gt
       }
       Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
@@ -462,11 +496,11 @@ if(is.null(starting) & is.null(n.combi)){
 
     #Sum of squared error to be optimized
     ssq1 <- function(param){
-      gamma <- param[1:((m-1)*ncoly)]
-      c <- param[(ncoly*(m-1)+1):length(param)]
+      gamma <- param[1:((m-1)*ny)]
+      c <- param[(ny*(m-1)+1):length(param)]
       gamma1 <- matrix(gamma, ncol = (m-1))
       c1 <- matrix(c, ncol = (m-1))
-      glog <- matrix(ncol=ncoly, nrow = nrowy)
+      glog <- matrix(ncol=ny, nrow = nrowy)
       GT <- list()
       Gtilde <- list()
       dify <- matrix(ncol = 1, nrow = nrow(y))
@@ -474,11 +508,15 @@ if(is.null(starting) & is.null(n.combi)){
       In <- diag(ncoly)
       for (z in 1:nrow(y)){
         for(t in 1:(m-1)){
-          for (o in 1:ncoly){
+          for (o in 1:ny){
             gammao <- gamma1[o,t]
             co <- c1[o,t]
             glog[z,o] <- (1L+exp(-gammao*(st[z]-co)))^(-1)}
-          Gt <- diag(glog[z,])
+          if(singlecgamma == TRUE){
+            Gt <- diag(rep(glog[z,1], ncoly))
+          }else{
+            Gt <- diag(glog[z,])
+          }
           GT[[t]] <- Gt
         }
         Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
@@ -490,11 +528,11 @@ if(is.null(starting) & is.null(n.combi)){
 
     #Maximum likelihood calculation
     loglike1 <- function(BB, param, Omega){
-      gamma <- param[1:((m-1)*ncoly)]
-      c <- param[(ncoly*(m-1)+1):length(param)]
+      gamma <- param[1:((m-1)*ny)]
+      c <- param[(ny*(m-1)+1):length(param)]
       gamma1 <- matrix(gamma, ncol = (m-1))
       c1 <- matrix(c, ncol = (m-1))
-      glog <- matrix(ncol=ncoly, nrow = nrowy)
+      glog <- matrix(ncol=ny, nrow = nrowy)
       GT <- list()
       Gtilde <- list()
       dify <- matrix(ncol = 1, nrow = nrow(y))
@@ -502,18 +540,22 @@ if(is.null(starting) & is.null(n.combi)){
       In <- diag(ncoly)
       for (z in 1:nrow(y)){
         for(t in 1:(m-1)){
-          for (o in 1:ncoly){
+          for (o in 1:ny){
             gammao <- gamma[o]
             co <- c[o]
             glog[z,o] <- (1L+exp(-gammao*(st[z]-co)))^(-1)}
-          Gt <- diag(glog[z,])
+          if(singlecgamma == TRUE){
+            Gt <- diag(rep(glog[z,1], ncoly))
+          }else{
+            Gt <- diag(glog[z,])
+          }
           GT[[t]] <- Gt
         }
         Gtilde[[z]] <- t(cbind(In, list.cbind(GT)))
-        dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omegahat)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
+        dify[z] <-  t(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])%*%ginv(Omega)%*%(y[z, ] - t(Gtilde[[z]])%*%t(BB)%*%x[z,])
       }
       sumdif <- sum(dify)
-      logll <- -(nrowy*log(det(Omegahat))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
+      logll <- -(nrowy*log(det(Omega))/2L) - sumdif/2L  - (nrowy*ncoly/2L)*log(2L*pi)
       return(-logll)
     }
 
@@ -544,15 +586,15 @@ if(is.null(starting) & is.null(n.combi)){
       Sys.sleep(0)
       iter <- iter+1L
       #Parameters
-      low1 <- replicate(ncoly, 0)
+      low1 <- replicate(ny, 0)
       #1.Maximum likelihood estimation of gamma and c
       param1 <- optim(par = param, fn = ssq1, lower = c(low1, apply(y, 2, min)),
-                             method="L-BFGS-B")
+                      method="L-BFGS-B")
 
-      cgam1 <- matrix(param1$par, ncol = 2L, nrow = ncoly*(m-1))
+      cgam1 <- matrix(param1$par, ncol = 2L, nrow = (ny*(m-1)))
 
       #2.NLS estimation of Bhat with new values of gamma and c
-      glog <- matrix(ncol=ncoly, nrow = nrowy)
+      glog <- matrix(ncol=ny, nrow = nrowy)
       GT <- list()
       Gtilde <- list()
       GG <- list()
@@ -564,10 +606,14 @@ if(is.null(starting) & is.null(n.combi)){
       ggxx <- matrix(ncol = (q+ncoly)*m*ncoly, nrow = (q+ncoly)*m*ncoly)
       for (i in 1:nrowx){
         for(t in 1:(m-1)){
-          for (j in 1 : ncoly){
-            glog[i,j] <- (1+exp(-cgam1[((t-1)*ncoly + j),1]*(st[i]-cgam1[((t-1)*ncoly + j),2])))^(-1)
+          for (j in 1 : ny){
+            glog[i,j] <- (1+exp(-cgam1[((t-1)*ny + j),1]*(st[i]-cgam1[((t-1)*ny + j),2])))^(-1)
+          }
+          if(singlecgamma == TRUE){
+            Gt <- diag(rep(glog[i,1], ncoly))
+          }else{
+            Gt <- diag(glog[i,])
           }
-          Gt <- diag(glog[i,])
           GT[[t]] <- Gt
         }
         Gtilde[[i]] <- t(cbind(In, list.cbind(GT)))
@@ -695,20 +741,20 @@ if(is.null(starting) & is.null(n.combi)){
     colnames(bhat1) <- colnames(y)
     modeldata <- list(y, x)
     fitte <- fitte[!is.na(fitte[,1]),]
-    results <- list(BBhat, covbb, ttest, pval, cgam1, omega[[iter]], fitte, residuals1, ll1, ll2, AIC1, BIC1, Gt, modeldata, BB, m, p,
-                    st, y, exo, constant)
+    results <- list(BBhat, covbb, ttest, pval, cgam1, omega[[iter]], fitte, residuals1, ll1, ll2, AIC1, BIC1, Gtilde, modeldata, BB, m, p,
+                    st, y, exo, constant, method, singlecgamma)
     names(results) <- c('Bhat','StDev', 'ttest', 'pval', 'Gammac', 'Omega', 'fitted', 'residuals', 'MultiLL', 'LL', 'AIC',
-                        'BIC', 'Gtilde', 'Data', 'B', 'm', 'p', 'st', 'yoriginal', 'exo', 'constant')
+                        'BIC', 'Gtilde', 'Data', 'B', 'm', 'p', 'st', 'yoriginal', 'exo', 'constant', 'method', 'singlecgamma')
 
   }
-  class(results) = 'VLSTAR'
-  return(results)
+class(results) = 'VLSTAR'
+return(results)
 }
 
-#' @S3method print VLSTARnls
+#' @S3method print VLSTAR
 print.VLSTAR <- function(x, ...) {
   digits = 3
-  cat("\nVLSTAR model Estimation through Nonlinear Least Squares\n")
+  cat(paste("\nVLSTAR model Estimation through", x$method, "\n"))
   object <- x
   order.L <- (object$m-1)
   order.H <- object$m
@@ -719,9 +765,9 @@ print.VLSTAR <- function(x, ...) {
 
   cat("Coefficients:\n")
   cat("Low regime:\n")
-  print(lowCoef, ...)
+  print(lowCoef)
   cat("\nHigh regime:\n")
-  print(highCoef, ...)
+  print(highCoef)
   cat("\nSmoothing parameter: gamma =", format(gammaCoef, digits=4),"\n")
   cat("\nThreshold")
   cat("\nValue:", format(cCoef, digits=4), "\n")
