@@ -9,6 +9,7 @@ predict.VLSTAR <- function(object, ..., n.ahead = 1, conf.lev = 0.95, st.new = N
   nrowy <- nrow(object$Data[[1]])
   alpha = 1-conf.lev
   k <- ncol(object$exo)
+  In <- diag(ncol(object$Data[[1]]))
   datamat <- object$Data[[2]]
 if(is.null(object$exo)){
   if(object$constant == T){
@@ -34,11 +35,10 @@ newdata <- as.matrix(newdata)
   BB <- object$B
   BB1 <- object$B[,seq(1, ncol(BB))[c(rep(TRUE, ncoly), rep(FALSE, ncoly))]]
   BB2 <- object$B[,seq(1, ncol(BB))[c(rep(FALSE, ncoly), rep(TRUE, ncoly))]]
-  In <- diag(ncol(object$Data[[1]]))
-  Gtilde <- t(cbind(In, object$Gtilde))
+  Gtilde <- t(object$Gtilde[[length(object$Gtilde)]])
   pred <- matrix(NA, ncol = ncoly, nrow = n.ahead)
   Z <- c(constant[1, ], yy, newdata)
-  pred[1,] <- as.matrix((t(Gtilde)%*%t(BB)%*%Z))
+  pred[1,] <- as.matrix((Gtilde%*%t(BB)%*%Z))
   tmp <- pred[1, ]
   yy <- c(tmp)
   if(is.null(st.new)){
